# -*- coding:utf-8 -*-
#
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import sys
if sys.version_info >= (2, 7):
    import unittest
else:
    import unittest2 as unittest

from nazca.ner import NerProcess
from nazca.ner.filters import (NerOccurenceFilter,
                               NerReplacementRulesFilter,
                               NerDisambiguationWordParts)
from nazca.ner.sources import NerSourceLexicon
from nazca.utils.tokenizer import Token, Sentence, NLTK_AVAILABLE


@unittest.skipUnless(NLTK_AVAILABLE, 'nltk is not available')
class FilterTest(unittest.TestCase):
    """ Test of filters """

    def test_occurence_filter_min_occ(self):
        """ Test occurence filter """
        text = 'Hello everyone, this is   me speaking. And me.'
        source1 = NerSourceLexicon({'everyone': 'http://example.com/everyone',
                                    'me': 'http://example.com/me'})
        source2 = NerSourceLexicon({'me': 'http://example2.com/me'})
        _filter = NerOccurenceFilter(min_occ=2)
        ner = NerProcess((source1, source2), filters=(_filter,))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example2.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46))),
                          ('http://example2.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46)))])

    def test_occurence_filter_max_occ(self):
        """ Test occurence filter """
        text = 'Hello everyone, this is   me speaking. And me.'
        source1 = NerSourceLexicon({'everyone': 'http://example.com/everyone',
                                    'me': 'http://example.com/me'})
        source2 = NerSourceLexicon({'me': 'http://example2.com/me'})
        _filter = NerOccurenceFilter(max_occ=1)
        ner = NerProcess((source1, source2), filters=(_filter,))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/everyone', None,
                           Token(word='everyone', start=6, end=14,
                                           sentence=Sentence(indice=0, start=0, end=38))),])

    def test_disambiguation_word_length(self):
        """ Test occurence filter """
        text = 'Hello toto tutu. And toto.'
        source = NerSourceLexicon({'toto tutu': 'http://example.com/toto_tutu',
                                   'toto': 'http://example.com/toto'})
        _filter = NerDisambiguationWordParts()
        ner = NerProcess((source,), filters=(_filter,))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/toto_tutu', None,
                           Token(word='toto tutu', start=6, end=15,
                                 sentence=Sentence(indice=0, start=0, end=16))),
                          ('http://example.com/toto_tutu', None,
                           Token(word='toto', start=21, end=25,
                                 sentence=Sentence(indice=1, start=17, end=26)))])

    def test_disambiguation_word_case(self):
        """ Test occurence filter """
        text = 'Hello Toto Tutu. And Toto.'
        source = NerSourceLexicon({'Toto Tutu': 'http://example.com/toto_tutu',
                                   'Toto': 'http://example.com/toto'})
        _filter = NerDisambiguationWordParts()
        ner = NerProcess((source,), filters=(_filter,))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/toto_tutu', None,
                           Token(word='Toto Tutu', start=6, end=15,
                                 sentence=Sentence(indice=0, start=0, end=16))),
                          ('http://example.com/toto_tutu', None,
                           Token(word='Toto', start=21, end=25,
                                 sentence=Sentence(indice=1, start=17, end=26)))])

    def test_rules_filter(self):
        """ Test rules filter """
        text = 'Hello toto tutu. And toto.'
        source = NerSourceLexicon({'toto tutu': 'http://example.com/toto_tutu',
                                   'toto': 'http://example.com/toto'})
        rules = {'http://example.com/toto': 'http://example.com/tata'}
        _filter = NerReplacementRulesFilter(rules)
        ner = NerProcess((source,), filters=(_filter,))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/toto_tutu', None,
                           Token(word='toto tutu', start=6, end=15,
                                 sentence=Sentence(indice=0, start=0, end=16))),
                          ('http://example.com/tata', None,
                           Token(word='toto', start=21, end=25,
                                 sentence=Sentence(indice=1, start=17, end=26)))])

if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-

# See http://en.wikipedia.org/wiki/List_of_U.S._state_abbreviations
# WARNING: The name of each state should be in French
# (e.g. "Floride", not "Florida")
US_STATES = {'AK': 'Alaska',
             'AL': 'Alabama',
             'AR': 'Arkansas',
             'AZ': 'Arizona',
             'Ala.': 'Alabama',
             'Alas.': 'Alaska',
             'Alaska': 'Alaska',
             'Ariz.': 'Arizona',
             'Ark.': 'Arkansas',
             'Az.': 'Arizona',
             'CA': 'Californie',
             'CF': 'Californie',
             'CL': 'Colorado',
             'CO': 'Colorado',
             'CT': 'Connecticut',
             'Ca.': 'Californie',
             'Cal.': 'Californie',
             'Cali.': 'Californie',
             'Calif.': 'Californie',
             'Col.': 'Colorado',
             'Colo.': 'Colorado',
             'Conn.': 'Connecticut',
             'Ct.': 'Connecticut',
             'D.C.': 'District of ColuFederal district',
             'DC': 'District of ColuFederal district',
             'DE': 'Delaware',
             'DL': 'Delaware',
             'De.': 'Delaware',
             'Del.': 'Delaware',
             'FL': 'Floride',
             'Fl.': 'Floride',
             'Fla.': 'Floride',
             'Flor.': 'Floride',
             'GA': u'Géorgie',
             'Ga.': u'Géorgie',
             'H.I.': 'Hawaii',
             'HA': 'Hawaii',
             'HI': 'Hawaii',
             'Hawaii': 'Hawaii',
             'IA': 'Iowa',
             'ID': 'Idaho',
             'IL': 'Illinois',
             'IN': 'Indiana',
             'Ia.': 'Iowa',
             'Id.': 'Idaho',
             'Ida.': 'Idaho',
             'Idaho': 'Idaho',
             'Il.': 'Illinois',
             "Ill's": 'Illinois',
             'Ill.': 'Illinois',
             'Ills.': 'Illinois',
             'In.': 'Indiana',
             'Ind.': 'Indiana',
             'Ioa.': 'Iowa',
             'Iowa': 'Iowa',
             'KA': 'Kansas',
             'KS': 'Kansas',
             'KY': 'Kentucky',
             'Ka.': 'Kansas',
             'Kan.': 'Kansas',
             'Kans.': 'Kansas',
             'Ks.': 'Kansas',
             'Ky.': 'Kentucky',
             'LA': 'Louisiane',
             'La.': 'Louisiane',
             'MA': 'Massachusetts',
             'MC': 'Michigan',
             'MD': 'Maryland',
             'ME': 'Maine',
             'MI': 'Mississippi',
             'MN': 'Minnesota',
             'MO': 'Missouri',
             'MS': 'Mississippi',
             'MT': 'Montana',
             'Maine': 'Maine',
             'Mass.': 'Massachusetts',
             'Md.': 'Maryland',
             'Me.': 'Maine',
             'Mich.': 'Michigan',
             'Minn.': 'Minnesota',
             'Miss.': 'Mississippi',
             'Mn.': 'Minnesota',
             'Mo.': 'Missouri',
             'Mont.': 'Montana',
             'N. Car.': 'Caroline du Nord',
             'N. Dak.': 'Dakota du Nord',
             'N. Mex.': 'Nouveau-Mexique',
             'N. York': 'New York',
             'N.C.': 'Caroline du Nord',
             'N.D.': 'Dakota du Nord',
             'N.H.': 'New Hampshire',
             'N.J.': 'New Jersey',
             'N.M.': 'Nouveau-Mexique',
             'N.Y.': 'New York',
             'NB': 'Nebraska',
             'NC': 'Caroline du Nord',
             'ND': 'Dakota du Nord',
             'NE': 'Nebraska',
             'NH': 'New Hampshire',
             'NJ': 'New Jersey',
             'NM': 'Nouveau-Mexique',
             'NV': 'Nevada',
             'NY': 'New York',
             'Neb.': 'Nebraska',
             'Nebr.': 'Nebraska',
             'Nev.': 'Nevada',
             'New M.': 'Nouveau-Mexique',
             'NoDak': 'Dakota du Nord',
             'Nv.': 'Nevada',
             'O.': 'Ohio',
             'OH': 'Ohio',
             'OK': 'Oklahoma',
             'OR': 'Oregon',
             'Oh.': 'Ohio',
             'Ohio': 'Ohio',
             'Ok.': 'Oklahoma',
             'Okla.': 'Oklahoma',
             'Or.': 'Oregon',
             'Ore.': 'Oregon',
             'Oreg.': 'Oregon',
             'PA': 'Pennsylvanie',
             'Pa.': 'Pennsylvanie',
             'R.I.': 'Rhode Island',
             'R.I. & P.P.': 'Rhode Island',
             'RI': 'Rhode Island',
             'S. Car.': 'Caroline du Sud',
             'S. Dak.': 'Dakota du Sud',
             'S.C.': 'Caroline du Sud',
             'S.D.': 'Dakota du Sud',
             'SC': 'Caroline du Sud',
             'SD': 'Dakota du Sud',
             'SoDak': 'Dakota du Sud',
             'State': 'Utah',
             'TN': 'Tennessee',
             'TX': 'Texas',
             'Tenn.': 'Tennessee',
             'Tex.': 'Texas',
             'Texas': 'Texas',
             'Tn.': 'Tennessee',
             'Tx.': 'Texas',
             'US-AL': 'Alabama',
             'US-AR': 'Arkansas',
             'US-AZ': 'Arizona',
             'US-CA': 'Californie',
             'US-CO': 'Colorado',
             'US-CT': 'Connecticut',
             'US-DC': 'District of ColuFederal district',
             'US-DE': 'Delaware',
             'US-FL': 'Floride',
             'US-GA': u'Géorgie',
             'US-IL': 'Illinois',
             'US-IN': 'Indiana',
             'US-KY': 'Kentucky',
             'US-LA': 'Louisiane',
             'US-MA': 'Massachusetts',
             'US-MD': 'Maryland',
             'US-MI': 'Michigan',
             'US-MN': 'Minnesota',
             'US-MO': 'Missouri',
             'US-MS': 'Mississippi',
             'US-MT': 'Montana',
             'US-NC': 'Caroline du Nord',
             'US-ND': 'Dakota du Nord',
             'US-NE': 'Nebraska',
             'US-NH': 'New Hampshire',
             'US-NJ': 'New Jersey',
             'US-NM': 'Nouveau-Mexique',
             'US-NY': 'New York',
             'US-OK': 'Oklahoma',
             'US-PA': 'Pennsylvanie',
             'US-RI': 'Rhode Island',
             'US-SC': 'Caroline du Sud',
             'US-SD': 'Dakota du Sud',
             'US-TN': 'Tennessee',
             'US-VA': 'Virginia',
             'US-VT': 'Vermont',
             'US-WA': 'Washington',
             'US-WI': 'Wisconsin',
             'US-WV': 'Virginie occidentale',
             'US-WY': 'Wyoming',
             'UT': 'Utah',
             'Ut.': 'Utah',
             'Utah': 'Utah',
             'VA': 'Virginia',
             'VT': 'Vermont',
             'Va.': 'Virginia',
             'Vt.': 'Vermont',
             'W. Va.': 'Virginie occidentale',
             'W. Virg.': 'Virginie occidentale',
             'W.V.': 'Virginie occidentale',
             'W.Va.': 'Virginie occidentale',
             'WA': 'Washington',
             'WI': 'Wisconsin',
             'WN': 'Washington',
             'WS': 'Wisconsin',
             'WV': 'Virginie occidentale',
             'WY': 'Wyoming',
             'Wa.': 'Washington',
             'Wash.': 'Washington',
             'Wash. D.C.': 'District of ColuFederal district',
             'Wi.': 'Wisconsin',
             'Wis.': 'Wisconsin',
             'Wisc.': 'Wisconsin',
             'Wn.': 'Washington',
             'Wy.': 'Wyoming',
             'Wyo.': 'Wyoming'}



# -*- coding:utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
import time
import logging
from collections import defaultdict

from scipy import zeros
from scipy.sparse import lil_matrix

from nazca.utils.dataio import parsefile


###############################################################################
### UTILITY FUNCTIONS #########################################################
###############################################################################
def iter_aligned_pairs(refset, targetset, global_mat, global_matched, unique=True):
    """ Return the aligned pairs
    """
    if unique:
        for refid in global_matched:
            bestid, _ = sorted(global_matched[refid], key=lambda x:x[1])[0]
            ref_record = refset[refid]
            target_record = targetset[bestid]
            distance = global_mat[refid, bestid] if global_mat is not None else None
            yield (ref_record[0], refid), (target_record[0], bestid), distance
    else:
        for refid in global_matched:
            for targetid, _ in global_matched[refid]:
                ref_record = refset[refid]
                target_record = targetset[targetid]
                distance = global_mat[refid, targetid] if global_mat is not None else None
                yield (ref_record[0], refid), (target_record[0], targetid), distance


###############################################################################
### BASE ALIGNER OBJECT #######################################################
###############################################################################
class BaseAligner(object):

    def __init__(self, threshold, processings, normalize_matrix=False):
        self.threshold = threshold
        self.processings = processings
        self.normalize_matrix = normalize_matrix
        self.ref_normalizer = None
        self.target_normalizer = None
        self.target_normalizer = None
        self.blocking = None
        self.alignments_done = 0
        self.pairs_found = 0
        self.nb_comparisons = 0
        self.nb_blocks = 0
        self.refset_size = None
        self.targetset_size = None
        self.time = None
        self.logger = logging.getLogger('nazca.aligner')

    def register_ref_normalizer(self, normalizer):
        """ Register normalizers to be applied
        before alignment """
        self.ref_normalizer = normalizer

    def register_target_normalizer(self, normalizer):
        """ Register normalizers to be applied
        before alignment """
        self.target_normalizer = normalizer

    def register_blocking(self, blocking):
        self.blocking = blocking

    def apply_normalization(self, dataset, normalizer):
        if normalizer:
            return normalizer.normalize_dataset(dataset)
        return dataset

    def compute_distance_matrix(self, refset, targetset,
                                ref_indexes, target_indexes):
        """ Compute and return the global alignment matrix.
        For each `processing` a `Distancematrix` is built, then all the
        matrices are summed with their own weighting and the result is the global
        alignment matrix, which is returned.
        """
        distmatrix = zeros((len(ref_indexes), len(target_indexes)), dtype='float32')
        for processing in self.processings:
            distmatrix += processing.cdist(refset, targetset,
                                          ref_indexes, target_indexes)
        return distmatrix

    def threshold_matched(self, distmatrix):
        """ Return the matched elements within a dictionnary,
        each key being the indice from X, and the corresponding
        values being a list of couple (indice from Y, distance)
        """
        match = defaultdict(list)
        if self.normalize_matrix:
            distmatrix /= distmatrix.max()
        ind = (distmatrix <= self.threshold).nonzero()
        indrow = ind[0].tolist()
        indcol = ind[1].tolist()
        for (i, j) in zip(indrow, indcol):
            match[i].append((j, distmatrix[i, j]))
        return match

    def _get_match(self, refset, targetset, ref_indexes=None, target_indexes=None):
        # Build items
        items = []
        ref_indexes = ref_indexes or xrange(len(refset))
        target_indexes = target_indexes or xrange(len(targetset))
        # Apply alignments
        mat = self.compute_distance_matrix(refset, targetset,
                                           ref_indexes=ref_indexes,
                                           target_indexes=target_indexes)
        matched = self.threshold_matched(mat)
        # Reapply matched to global indexes
        new_matched = {}
        for k, values in matched.iteritems():
            new_matched[ref_indexes[k]] = [(target_indexes[i], d) for i, d in values]
        return mat, new_matched

    def align(self, refset, targetset, get_matrix=True):
        """ Perform the alignment on the referenceset
        and the targetset
        """
        start_time = time.time()
        refset = self.apply_normalization(refset, self.ref_normalizer)
        targetset = self.apply_normalization(targetset, self.target_normalizer)
        self.refset_size = len(refset)
        self.targetset_size = len(targetset)
        # If no blocking
        if not self.blocking:
            return self._get_match(refset, targetset)
        # Blocking == conquer_and_divide
        global_matched = {}
        global_mat = lil_matrix((len(refset), len(targetset)))
        self.blocking.fit(refset, targetset)
        for refblock, targetblock in self.blocking.iter_blocks():
            self.nb_blocks += 1
            ref_index = [r[0] for r in refblock]
            target_index = [r[0] for r in targetblock]
            self.nb_comparisons += len(ref_index)*len(target_index)
            _, matched = self._get_match(refset, targetset, ref_index, target_index)
            for k, values in matched.iteritems():
                subdict = global_matched.setdefault(k, set())
                for v, d in values:
                    subdict.add((v, d))
                    self.alignments_done += 1
                    if get_matrix:
                        # XXX avoid issue in sparse matrix
                        global_mat[k, v] = d or 10**(-10)
        self.time = time.time() - start_time
        return global_mat, global_matched

    def get_aligned_pairs(self, refset, targetset, unique=True, use_distance=True):
        """ Get the pairs of aligned elements
        """
        global_mat, global_matched = self.align(refset, targetset, get_matrix=use_distance)
        for pair in iter_aligned_pairs(refset, targetset, global_mat, global_matched, unique):
            self.pairs_found += 1
            yield pair
        self.log_infos()

    def align_from_files(self, reffile, targetfile,
                         ref_indexes=None, target_indexes=None,
                         ref_encoding=None, target_encoding=None,
                         ref_separator='\t', target_separator='\t',
                         get_matrix=True):
        """ Align data from files

        Parameters
        ----------

        reffile: name of the reference file

        targetfile: name of the target file

        ref_encoding: if given (e.g. 'utf-8' or 'latin-1'), it will
                      be used to read the files.

        target_encoding: if given (e.g. 'utf-8' or 'latin-1'), it will
                         be used to read the files.

        ref_separator: separator of the reference file

        target_separator: separator of the target file
        """
        refset = parsefile(reffile, indexes=ref_indexes,
                           encoding=ref_encoding, delimiter=ref_separator)
        targetset = parsefile(targetfile, indexes=target_indexes,
                              encoding=target_encoding, delimiter=target_separator)
        return self.align(refset, targetset, get_matrix=get_matrix)

    def get_aligned_pairs_from_files(self, reffile, targetfile,
                         ref_indexes=None, target_indexes=None,
                         ref_encoding=None, target_encoding=None,
                         ref_separator='\t', target_separator='\t',
                         unique=True):
        """ Get the pairs of aligned elements
        """
        refset = parsefile(reffile, indexes=ref_indexes,
                           encoding=ref_encoding, delimiter=ref_separator)
        targetset = parsefile(targetfile, indexes=target_indexes,
                              encoding=target_encoding, delimiter=target_separator)
        global_mat, global_matched = self.align(refset, targetset, get_matrix=False)
        for pair in iter_aligned_pairs(refset, targetset, global_mat, global_matched, unique):
            yield pair

    def log_infos(self):
        """ Display some info on the aligner process
        """
        self.logger.info('Computation time : %s' % self.time)
        self.logger.info('Size reference set : %s' % self.refset_size)
        self.logger.info('Size target set : %s' % self.targetset_size)
        self.logger.info('Comparisons done : %s' % self.nb_comparisons)
        self.logger.info('Alignments done : %s' % self.alignments_done)
        self.logger.info('Pairs found : %s' % self.pairs_found)
        self.logger.info('Ratio reference set/alignments done : %s'
                         % (self.alignments_done/float(self.refset_size)))
        self.logger.info('Ratio target set/alignments done : %s'
                         % (self.alignments_done/float(self.targetset_size)))
        self.logger.info('Ratio reference set/pairs found : %s'
                         % (self.pairs_found/float(self.refset_size)))
        self.logger.info('Ratio target set/pairs found : %s'
                         % (self.pairs_found/float(self.targetset_size)))
        self.logger.info('Maximum comparisons : %s'
                         % (self.refset_size * self.targetset_size))
        self.logger.info('Number of blocks : %s' % self.nb_blocks)
        if self.nb_blocks:
            self.logger.info('Ratio comparisons/block : %s'
                             % (float(self.nb_comparisons)/self.nb_blocks))
        self.logger.info('Blocking reduction : %s'
                         % (self.nb_comparisons/float(self.refset_size * self.targetset_size)))


###############################################################################
### PIPELINE ALIGNER OBJECT ##################################################
###############################################################################
class PipelineAligner(object):
    """ This pipeline will perform iterative alignments, removing each time
    the aligned results from the previous aligner.
    """

    def __init__(self, aligners):
        self.aligners = aligners
        self.pairs = {}
        self.nb_comparisons = 0
        self.nb_blocks = 0
        self.alignments_done = 0
        self.pairs_found = 0
        self.refset_size = None
        self.targetset_size = None
        self.time = None
        self.logger = logging.getLogger('nazca.aligner')

    def get_aligned_pairs(self, refset, targetset, unique=True):
        """ Get the pairs of aligned elements
        """
        start_time = time.time()
        ref_index = range(len(refset))
        target_index = range(len(targetset))
        self.refset_size = len(refset)
        self.targetset_size = len(targetset)
        global_matched = {}
        global_mat = lil_matrix((len(refset), len(targetset)))
        seen_refset = set()
        # Iteration over aligners
        for ind_aligner, aligner in enumerate(self.aligners):
            # Perform alignment
            _refset = [refset[i] for i in ref_index]
            _targetset = [targetset[i] for i in target_index]
            for pair in aligner.get_aligned_pairs(_refset, _targetset, unique):
                self.pairs_found += 1
                pair = ((pair[0][0], ref_index[pair[0][1]]),
                        (pair[1][0], target_index[pair[1][1]]))
                yield pair
                seen_refset.add(pair[0][1])
            # Store stats
            self.nb_blocks += aligner.nb_blocks
            self.nb_comparisons += aligner.nb_comparisons
            # Update indexes if necessary
            # For now, we remove all the reference set that are already matched
            if ind_aligner < len(self.aligners) - 1:
                # There are other aligners after this one
                ref_index = [i for i in ref_index if i not in seen_refset]
        self.time = time.time() - start_time
        self.log_infos()

    def log_infos(self):
        """ Display some info on the aligner process
        """
        self.logger.info('Computation time : %s' % self.time)
        self.logger.info('Size reference set : %s' % self.refset_size)
        self.logger.info('Size target set : %s' % self.targetset_size)
        self.logger.info('Comparisons done : %s' % self.nb_comparisons)
        self.logger.info('Alignments done : %s' % self.alignments_done)
        self.logger.info('Pairs found : %s' % self.pairs_found)
        self.logger.info('Ratio reference set/alignments done : %s'
                         % (self.alignments_done/float(self.refset_size)))
        self.logger.info('Ratio target set/alignments done : %s'
                         % (self.alignments_done/float(self.targetset_size)))
        self.logger.info('Ratio reference set/pairs found : %s'
                         % (self.pairs_found/float(self.refset_size)))
        self.logger.info('Ratio target set/pairs found : %s'
                         % (self.pairs_found/float(self.targetset_size)))
        self.logger.info('Maximum comparisons : %s'
                         % (self.refset_size * self.targetset_size))
        self.logger.info('Number of blocks : %s' % self.nb_blocks)
        if self.nb_blocks:
            self.logger.info('Ratio comparisons/block : %s'
                             % (float(self.nb_comparisons)/self.nb_blocks))
        self.logger.info('Blocking reduction : %s'
                         % (self.nb_comparisons/float(self.refset_size * self.targetset_size)))



# -*- coding:utf-8 -*-
#
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import sys
if sys.version_info >= (2, 7):
    import unittest
else:
    import unittest2 as unittest

from nazca.ner.sources import (NerSourceLexicon,
                               NerSourceSparql,
                               NerSourceRql)
from nazca.ner import NerProcess
from nazca.utils.tokenizer import Token, Sentence, NLTK_AVAILABLE
from nazca.ner.preprocessors import NerStopwordsFilterPreprocessor
from nazca.utils.dataio import SPARQL_ENABLED


class NerTest(unittest.TestCase):
    """ Test of Ner """

    def test_lexicon_source(self):
        """ Test lexicon source """
        lexicon = {'everyone': 'http://example.com/everyone',
                   'me': 'http://example.com/me'}
        source = NerSourceLexicon(lexicon)
        self.assertEqual(source.query_word('me'), ['http://example.com/me',])
        self.assertEqual(source.query_word('everyone'), ['http://example.com/everyone',])
        self.assertEqual(source.query_word('me everyone'), [])
        self.assertEqual(source.query_word('toto'), [])
        # Token
        token = Token('me', 0, 2, None)
        self.assertEqual(source.recognize_token(token), ['http://example.com/me',])
        token = Token('ma', 0, 2, None)
        self.assertEqual(source.recognize_token(token), [])

    def test_rql_source(self):
        """ Test rql source """
        source = NerSourceRql('http://www.cubicweb.org',
                              'Any U LIMIT 1 WHERE X cwuri U, X name "%(word)s"')
        self.assertEqual(source.query_word('apycot'), [u'http://www.cubicweb.org/1310453',])

    @unittest.skipUnless(SPARQL_ENABLED, 'python-sparqlwrapper is not installed')
    def test_sparql_source(self):
        """ Test sparql source """
        source = NerSourceSparql(u'http://dbpedia.org/sparql',
                             u'''SELECT DISTINCT ?uri
                                 WHERE {
                                 ?uri rdf:type <http://dbpedia.org/ontology/ProgrammingLanguage> ;
                                      dbpedia-owl:designer <http://dbpedia.org/resource/Guido_van_Rossum> ;
                                      rdfs:label ?label.
                                 FILTER(regex(?label, "^%(word)s"))
                                 }''')
        self.assertEqual(source.query_word('Python'),
                         ['http://dbpedia.org/resource/Python_(programming_language)',
                          'http://dbpedia.org/resource/Python_for_S60'])

    @unittest.skipUnless(NLTK_AVAILABLE, 'nltk is not available')
    def test_ner_process(self):
        """ Test ner process """
        text = 'Hello everyone, this is   me speaking. And me.'
        source = NerSourceLexicon({'everyone': 'http://example.com/everyone',
                                   'me': 'http://example.com/me'})
        ner = NerProcess((source,))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/everyone', None,
                           Token(word='everyone', start=6, end=14,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46)))])

    @unittest.skipUnless(NLTK_AVAILABLE, 'nltk is not available')
    def test_ner_process_multisources(self):
        """ Test ner process """
        text = 'Hello everyone, this is   me speaking. And me.'
        source1 = NerSourceLexicon({'everyone': 'http://example.com/everyone',
                                    'me': 'http://example.com/me'})
        source2 = NerSourceLexicon({'me': 'http://example2.com/me'})
        # Two sources, not unique
        ner = NerProcess((source1, source2))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/everyone', None,
                           Token(word='everyone', start=6, end=14,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example2.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46))),
                          ('http://example2.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46)))])
        # Two sources, unique
        ner = NerProcess((source1, source2), unique=True)
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/everyone', None,
                           Token(word='everyone', start=6, end=14,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46)))])
        # Two sources inversed, unique
        ner = NerProcess((source2, source1), unique=True)
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/everyone', None,
                           Token(word='everyone', start=6, end=14,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example2.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example2.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46)))])

    @unittest.skipUnless(NLTK_AVAILABLE, 'nltk is not available')
    def test_ner_process_add_sources(self):
        """ Test ner process """
        text = 'Hello everyone, this is   me speaking. And me.'
        source1 = NerSourceLexicon({'everyone': 'http://example.com/everyone',
                                    'me': 'http://example.com/me'})
        source2 = NerSourceLexicon({'me': 'http://example2.com/me'})
        ner = NerProcess((source1,))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/everyone', None,
                           Token(word='everyone', start=6, end=14,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46))),])
        # Two sources, not unique
        ner.add_ner_source(source2)
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/everyone', None,
                           Token(word='everyone', start=6, end=14,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example2.com/me', None,
                           Token(word='me', start=26, end=28,
                                           sentence=Sentence(indice=0, start=0, end=38))),
                          ('http://example.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46))),
                          ('http://example2.com/me', None,
                           Token(word='me', start=43, end=45,
                                           sentence=Sentence(indice=1, start=39, end=46)))])

    @unittest.skipUnless(NLTK_AVAILABLE, 'nltk is not available')
    def test_ner_process_preprocess(self):
        """ Test ner process """
        text = 'Hello Toto, this is   me speaking. And me.'
        source = NerSourceLexicon({'Toto': 'http://example.com/toto',
                                   'me': 'http://example.com/me'})
        preprocessor = NerStopwordsFilterPreprocessor()
        ner = NerProcess((source,),
                                  preprocessors=(preprocessor,))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities, [('http://example.com/toto', None,
                                           Token(word='Toto', start=6, end=10,
                                                 sentence=Sentence(indice=0, start=0, end=34)))])

    @unittest.skipUnless(NLTK_AVAILABLE, 'nltk is not available')
    def test_ner_process_add_preprocess(self):
        """ Test ner process """
        text = 'Hello Toto, this is   me speaking. And me.'
        source = NerSourceLexicon({'Toto': 'http://example.com/toto',
                                   'me': 'http://example.com/me'})
        preprocessor = NerStopwordsFilterPreprocessor()
        ner = NerProcess((source,),)
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/toto', None,
                           Token(word='Toto', start=6, end=10,
                                 sentence=Sentence(indice=0, start=0, end=34))),
                          ('http://example.com/me', None,
                           Token(word='me', start=22, end=24,
                                 sentence=Sentence(indice=0, start=0, end=34))),
                          ('http://example.com/me', None,
                           Token(word='me', start=39, end=41,
                                 sentence=Sentence(indice=1, start=35, end=42)))])
        ner.add_preprocessors(preprocessor)
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities, [('http://example.com/toto', None,
                                           Token(word='Toto', start=6, end=10,
                                                 sentence=Sentence(indice=0, start=0, end=34)))])

    @unittest.skipUnless(NLTK_AVAILABLE, 'nltk is not available')
    def test_ner_process_chained_word(self):
        """ Test ner process """
        text = 'Hello everyone me, this is   me speaking. And me.'
        source = NerSourceLexicon({'everyone': 'http://example.com/everyone',
                                   'everyone me': 'http://example.com/everyone_me',
                                   'me': 'http://example.com/me'})
        ner = NerProcess((source,))
        named_entities = ner.process_text(text)
        self.assertEqual(named_entities,
                         [('http://example.com/everyone_me', None,
                           Token(word='everyone me', start=6, end=17,
                                 sentence=Sentence(indice=0, start=0, end=41))),
                          ('http://example.com/me', None,
                           Token(word='me', start=29, end=31,
                                 sentence=Sentence(indice=0, start=0, end=41))),
                          ('http://example.com/me', None,
                           Token(word='me', start=46, end=48,
                                 sentence=Sentence(indice=1, start=42, end=49)))])


if __name__ == '__main__':
    unittest.main()




# -*- coding:utf-8 -*-
#
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import sys
if sys.version_info >= (2, 7):
    import unittest
else:
    import unittest2 as unittest
from os import path

from nazca.utils.normalize import (BaseNormalizer, UnicodeNormalizer, JoinNormalizer,
                                   SimplifyNormalizer, TokenizerNormalizer,
                                   LemmatizerNormalizer, RoundNormalizer,
                                   RegexpNormalizer, NormalizerPipeline,
                                   lunormalize, lemmatized,
                                   roundstr, rgxformat, tokenize, simplify)
from nazca.data import FRENCH_LEMMAS


TESTDIR = path.dirname(__file__)


class NormalizerFunctionTestCase(unittest.TestCase):

    def test_unormalize(self):
        self.assertEqual(lunormalize(u'bépoèàÀêùï'),
                                     u'bepoeaaeui')

    def test_simplify(self):
        self.assertEqual(simplify(u"J'aime les frites, les pommes et les" \
                                  u" scoubidous !", FRENCH_LEMMAS),
                         u"aimer frites pomme scoubidou")

    def test_tokenize(self):
        self.assertEqual(tokenize(u"J'aime les frites !"),
                         [u"J'", u'aime', u'les', u'frites', u'!',])

    def test_lemmatizer(self):
        self.assertEqual(lemmatized(u'sacré rubert', FRENCH_LEMMAS), u'sacré rubert')
        self.assertEqual(lemmatized(u"J'aime les frites !", FRENCH_LEMMAS),
                         u'je aimer le frite')
        self.assertEqual(lemmatized(u", J'aime les frites", FRENCH_LEMMAS),
                         u'je aimer le frite')

    def test_round(self):
        self.assertEqual(roundstr(3.14159, 2), '3.14')
        self.assertEqual(roundstr(3.14159), '3')
        self.assertEqual(roundstr('3.14159', 3), '3.142')

    def test_format(self):
        string = u'[Victor Hugo - 26 fev 1802 / 22 mai 1885]'
        regex  = r'\[(?P<firstname>\w+) (?P<lastname>\w+) - ' \
                 r'(?P<birthdate>.*) \/ (?P<deathdate>.*?)\]'
        output = u'%(lastname)s, %(firstname)s (%(birthdate)s - %(deathdate)s)'
        self.assertEqual(rgxformat(string, regex, output),
                         u'Hugo, Victor (26 fev 1802 - 22 mai 1885)')

        string = u'http://perdu.com/42/supertop/cool'
        regex  = r'http://perdu.com/(?P<id>\d+).*'
        output = u'%(id)s'
        self.assertEqual(rgxformat(string, regex, output),
                         u'42')


class NormalizerObjectTestCase(unittest.TestCase):

    def test_normalizer(self):
        normalizer = BaseNormalizer(lunormalize)
        self.assertEqual(normalizer.normalize(u'bépoèàÀêùï'), u'bepoeaaeui')

    def test_normalizer_record(self):
        normalizer = BaseNormalizer(lunormalize, attr_index=1)
        record = ('a1', u'bépoèàÀêùï')
        self.assertEqual(normalizer.normalize(record), ['a1',u'bepoeaaeui'])

    def test_normalizer_dataset(self):
        normalizer = BaseNormalizer(lunormalize, attr_index=1)
        dataset = [('a1', u'bépoèàÀêùï'), ('a2', u'tàtà')]
        results = normalizer.normalize_dataset(dataset)
        self.assertEqual([['a1', u'bepoeaaeui'], ['a2', u'tata']], results)
        self.assertNotEqual(results, dataset)

    def test_normalizer_dataset_inplace(self):
        normalizer = BaseNormalizer(lunormalize, attr_index=1)
        dataset = [('a1', u'bépoèàÀêùï'), ('a2', u'tàtà')]
        normalizer.normalize_dataset(dataset, inplace=True)
        self.assertEqual([['a1', u'bepoeaaeui'], ['a2', u'tata']], dataset)

    def test_unormalize(self):
        normalizer = UnicodeNormalizer()
        self.assertEqual(normalizer.normalize(u'bépoèàÀêùï'), u'bepoeaaeui')

    def test_unormalize_record(self):
        normalizer = UnicodeNormalizer(attr_index=1)
        record = ('a1', u'bépoèàÀêùï')
        self.assertEqual(['a1',u'bepoeaaeui'], normalizer.normalize(record))

    def test_simplify(self):
        normalizer = SimplifyNormalizer(lemmas=FRENCH_LEMMAS)
        self.assertEqual(normalizer.normalize(u"J'aime les frites, les pommes et les scoubidous !")
                         , u"aimer frites pomme scoubidou")

    def test_simplify_record(self):
        normalizer = SimplifyNormalizer(attr_index=1, lemmas=FRENCH_LEMMAS)
        self.assertEqual(['a1', u"aimer frites pomme scoubidou"],
                         normalizer.normalize(['a1', u"J'aime les frites, les pommes "
                                               "et les scoubidous !"]))

    def test_tokenize(self):
        normalizer = TokenizerNormalizer()
        self.assertEqual([u"J'", u'aime', u'les', u'frites', u'!',],
                         normalizer.normalize(u"J'aime les frites !"))

    def test_tokenize_record(self):
        normalizer = TokenizerNormalizer(attr_index=1)
        self.assertEqual(['a1', [u"J'", u'aime', u'les', u'frites', u'!',]],
                         normalizer.normalize(['a1', u"J'aime les frites !"]))

    def test_lemmatizer(self):
        normalizer = LemmatizerNormalizer(FRENCH_LEMMAS)
        self.assertEqual(normalizer.normalize(u'sacré rubert'), u'sacré rubert')
        self.assertEqual(normalizer.normalize(u"J'aime les frites !"), u'je aimer le frite')
        self.assertEqual(normalizer.normalize(u", J'aime les frites"), u'je aimer le frite')

    def test_lemmatizer_record(self):
        normalizer = LemmatizerNormalizer(FRENCH_LEMMAS, attr_index=1)
        self.assertEqual(['a1', u'sacré rubert'],
                         normalizer.normalize(['a1', u'sacré rubert']))
        self.assertEqual(['a1', u'je aimer le frite'],
                         normalizer.normalize(['a1', u"J'aime les frites !"]))
        self.assertEqual(['a1', u'je aimer le frite'],
                         normalizer.normalize(['a1', u", J'aime les frites"]))

    def test_round(self):
        normalizer = RoundNormalizer()
        self.assertEqual(normalizer.normalize(3.14159), '3')
        normalizer = RoundNormalizer(ndigits=2)
        self.assertEqual(normalizer.normalize(3.14159), '3.14')
        normalizer = RoundNormalizer(ndigits=3)
        self.assertEqual(normalizer.normalize(3.14159), '3.142')

    def test_round_record(self):
        normalizer = RoundNormalizer(attr_index=1)
        self.assertEqual(['a1', '3'], normalizer.normalize(['a1', 3.14159]))
        normalizer = RoundNormalizer(attr_index=1, ndigits=2)
        self.assertEqual(['a1', '3.14'], normalizer.normalize(['a1', 3.14159]))
        normalizer = RoundNormalizer(attr_index=1, ndigits=3)
        self.assertEqual(['a1', '3.142'], normalizer.normalize(['a1', 3.14159]))

    def test_regexp(self):
        normalizer = RegexpNormalizer(r'http://perdu.com/(?P<id>\d+).*', u'%(id)s')
        self.assertEqual(normalizer.normalize(u'http://perdu.com/42/supertop/cool'), u'42')

    def test_regexp_record(self):
        normalizer = RegexpNormalizer(r'http://perdu.com/(?P<id>\d+).*', u'%(id)s', attr_index=1)
        self.assertEqual(['a1', u'42'],
                         normalizer.normalize(['a1', u'http://perdu.com/42/supertop/cool']))

    def test_join(self):
        normalizer = JoinNormalizer((1,2))
        self.assertEqual(normalizer.normalize((1, 'ab', 'cd', 'e', 5)), [1, 'e', 5, 'ab, cd'])



class NormalizerPipelineTestCase(unittest.TestCase):

    def test_normalizer(self):
        regexp = r'(?P<id>\d+);{["]?(?P<firstname>.+[^"])["]?};{(?P<surname>.*)};{};{};(?P<date>.*)'
        output = u'%(id)s\t%(firstname)s\t%(surname)s\t%(date)s'
        n1 = RegexpNormalizer(regexp, u'%(id)s\t%(firstname)s\t%(surname)s\t%(date)s')
        n2 = BaseNormalizer(callback= lambda x: x.split('\t'))
        n3 = UnicodeNormalizer(attr_index=(1, 2, 3))
        pipeline = NormalizerPipeline((n1, n2, n3))
        r1 = u'1111;{"Toto tàtà"};{Titi};{};{};'
        self.assertEqual(['1111', 'toto tata', 'titi', u''], pipeline.normalize(r1))


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/python
#-*- coding:utf-8 -*-

from os import path

import urllib

import nazca.distances as ald
import nazca.normalize as aln
from nazca.aligner import align, subalign, findneighbours, alignall
from nazca.dataio import parsefile, sparqlquery, write_results

DEMODIR = path.dirname(__file__)

def dpath(filename):
    return path.join(DEMODIR, 'demo', filename)

def remove_after(string, sub):
    try:
        return string[:string.lower().index(sub)].strip()
    except ValueError:
        return string

def parserql(host, rql):
    filehandle = urllib.urlopen('%(host)sview?'
                                'rql=%(rql)s&vid=csvexport'
                                % {'rql': rql, 'host': host})
    filehandle.readline()
    rset = [[e.decode('utf-8') for e in line.strip().split(';')]
            for line in filehandle]
    return rset

def demo_0():
    # prixgoncourt is the list of Goncourt Prize, extracted
    # from wikipedia

    #We try to align Goncourt winers onto dbpedia results

    query = """
       SELECT ?writer, ?name WHERE {
          ?writer  <http://purl.org/dc/terms/subject> <http://dbpedia.org/resource/Category:French_novelists>.
          ?writer rdfs:label ?name.
          FILTER(lang(?name) = 'fr')
       }
    """

    print "Sending query to dbpedia"
    targetset = sparqlquery('http://dbpedia.org/sparql', query)
    print "Reading the prixgoncourt file"
    alignset = parsefile(dpath('prixgoncourt'), indexes=[1, 1])

    tr_name = {'normalization': [lambda x:remove_after(x, '('),
                                 aln.simplify],
               'metric': ald.levenshtein
              }

    processings = {1: tr_name}

    print "Alignment started"
    align(alignset, targetset, 0.4, processings,
          dpath('demo0_results'))

    print "Done, see the resuls in %s" % dpath('demo0_results')

def demo_1():
    # FR.txt is an extract of geonames, where locations have been sorted by name
    # frenchbnf is an extract of french BNF's locations, sorted by name too

    # For each line (ie location) we keep the identifier, the name and the
    # position (longitude, latitude)
    # ``nbmax`` is the number of locations to load

    print "Parsing the input files"
    targetset = parsefile(dpath('FR.txt'), indexes=[0, 1, (4, 5)],
                          nbmax=2000)
    alignset = parsefile(dpath('frenchbnf'),
                         indexes=[0, 2, (14, 12)], nbmax=1000)


    # Let's define the processings to apply on the location's name
    tr_name = {'normalization': [aln.simplify], # Simply all the names (remove
                                              #   punctuation, lower case, etc)
               'metric': ald.levenshtein,       # Use the levenshtein distance
               'weighting': 1                 # Use 1 a name-distance matrix
                                              #   weighting coefficient
              }
    tr_geo = {'normalization': [],              # No normalization needed
              'metric': ald.geographical,         # Use the geographical distance
              'metric_params': {'units': 'km'},# Arguments given the
                                                #   distance function. Here,
                                                #   the unit to use
              'weighting': 1
             }

    processings = {1: tr_name, 2: tr_geo}

    print "Alignment started"
    align(alignset,           # The dataset to align
          targetset,          # The target dataset
          0.4,                # The maximal distance
                              #   threshold
          processings,         # The list of processings to
                              #   apply.
          dpath('demo1_results'))
                              # Filename of the output
                              #   result file
    # the ``align()`` function return two items
    # 0. the computed distance matrix
    # 1. a boolean, True if at least one alignment has been done, False
    #    otherwise
    print "Done, see the results in %s" % dpath('demo1_results')

def demo_2():
    targetset = parsefile(dpath('FR.txt'), indexes=[0, 1, (4, 5)],
                          formatopt={1:lambda x:x.decode('utf-8')})
    alignset = parsefile(dpath('frenchbnf'), indexes=[0, 2, (14, 12)],
                         formatopt={2:lambda x:x.decode('utf-8')}, nbmax=30000)

    print "Finding neighbours"
    neighbours = findneighbours(alignset, targetset, indexes=(2, 2),
                               mode='minibatch')

    # Let's define the processings to apply on the location's name
    tr_name = {'normalization': [aln.simplify], # Simply all the names (remove
                                              #   punctuation, lower case, etc)
               'metric': ald.levenshtein,     # Use the levenshtein distance
               'weighting': 1                 # Use 1 a name-distance matrix
                                              #   weighting coefficient
              }

    processings = {1: tr_name}
    print "Start computation"
    for ind, (alignid, targetid) in enumerate(neighbours):
        print '%3d' % ind, len(alignid), 'x', len(targetid)
        _, matched = subalign(alignset,   # The dataset to align
                              targetset,  # The target dataset
                              alignid,
                              targetid,
                              0.3,
                              processings)
        write_results(matched, alignset, targetset, dpath('demo2_results'))
    print "Done, see the results in %s" % dpath('demo2_results')

def demo_3():
    print "Parsing files"
    alignset = parserql(host='http://demo.cubicweb.org/elections/',
                        rql='Any E, N WHERE X is Commune, X eid E, X label N')
    targetset = parsefile(dpath('FR.txt'), indexes=[0, 1])
    print '%s×%s' % (len(alignset), len(targetset))

    tr_name = {'normalization': [aln.simplify],
               'metric': 'levenshtein'
              }

    print "Alignment started"
    results = alignall(alignset, targetset, 0.75, processings={1: tr_name},
                       indexes=(1,1), mode='minhashing', kwordsgram=1, siglen=200,
                       uniq=True)
    dicresults = dict([(a, b) for (a, b) in results])

    print "Done, writing output"

    with open(dpath('demo3_results'), 'w') as fout:
        for line in alignset:
            sent = u'http://demo.cubicweb.org/elections/commune/%s;'\
                   u'http://www.geonames.org/%s\n' \
                   % (line[0], dicresults.get(line[0], 'not_found'))
            fout.write(sent.encode('utf-8'))

    print "See the results in %s" % dpath('demo3_results')

if __name__ == '__main__':
    import sys
    from time import time
    runall = (len(sys.argv) == 1)

    t = time()
    if runall or '0' in sys.argv:
        print "Running demo_0"
        demo_0()

    if runall or '1' in sys.argv:
        print "Running demo_1"
        demo_1()

    if runall or '2' in sys.argv:
        print "Running demo_2"
        ## Same as demo_1, but in a more efficient way, using a method to find
        ## neighbours
        demo_2()

    if runall or '3' in sys.argv:
        print "Running demo_3"
        demo_3()

    print "Demo terminated"
    print "Took %d min" % ((time() - t)/60.)



# -*- coding: utf-8 -*-
""" Data for Nazca
"""
import os.path as osp

HERE = osp.join(osp.abspath(osp.dirname(__file__)))
FRENCH_LEMMAS = dict([t.strip().split('\t') for t in open(osp.join(HERE, 'french_lemmas.txt'))])
